#pragma semicolon 1
#pragma newdecls required

#include <sourcemod>
#include <clientprefs>

#include <vip_core>

#define USE_SCP	// Использовать Simple Chat Processor (Redux)
//#define USE_CP	// Использовать Chat Processor

#if defined USE_SCP
	#include <scp>
#elseif defined USE_CP
	#include <chat-processor>
#else
	#error "Invalid chat processor!"
#endif

public Plugin myinfo =
{
	#if defined USE_SCP
	name = "[VIP] CHAT (scp)",
	#else
	name = "[VIP] CHAT (cp)",
	#endif
	author = "R1KO",
	version = "3.3"
};

EngineVersion g_EngineVersion;

enum
{
	PREFIX = 0,
	PREFIX_COLOR,
	NAME_COLOR,
	TEXT_COLOR,

	SIZE
};

char g_szFeature[][] = {"Chat", "Chat_Menu"},
	g_szCUSTOM[] = "custom",
	g_szLIST[] = "list",
	g_szFeatures[SIZE][] =
	{
		"Chat_Prefix",
		"Chat_PrefixColor",
		"Chat_NameColor",
		"Chat_TextColor"
	};

Handle g_hCookies[SIZE];
KeyValues g_hKeyValues;
ArrayList g_hIgnoredPhrases;
StringMap g_hColorsTrie;

bool g_bIgnoreTriggers,
	g_bIgnorePhrases,
	g_bReplaceOther;

bool g_bWaitChat[MAXPLAYERS+1];

public void VIP_OnVIPLoaded()
{
	VIP_RegisterFeature(g_szFeature[0], BOOL);
	VIP_RegisterFeature(g_szFeature[1], _, SELECTABLE, OnSelectItem, _, OnDrawItem);
	VIP_RegisterFeature(g_szFeatures[PREFIX], STRING, HIDE);
	VIP_RegisterFeature(g_szFeatures[PREFIX_COLOR], STRING, HIDE);
	VIP_RegisterFeature(g_szFeatures[NAME_COLOR], STRING, HIDE);
	VIP_RegisterFeature(g_szFeatures[TEXT_COLOR], STRING, HIDE);
}

public void OnPluginEnd()
{
	if(CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "VIP_UnregisterFeature") == FeatureStatus_Available)
	{
		VIP_UnregisterFeature(g_szFeature[0]);
		VIP_UnregisterFeature(g_szFeature[1]);
		VIP_UnregisterFeature(g_szFeatures[PREFIX]);
		VIP_UnregisterFeature(g_szFeatures[PREFIX_COLOR]);
		VIP_UnregisterFeature(g_szFeatures[NAME_COLOR]);
		VIP_UnregisterFeature(g_szFeatures[TEXT_COLOR]);
	}
}

public void OnPluginStart() 
{
	g_EngineVersion = GetEngineVersion();

	g_hColorsTrie = CreateTrie();

	g_hCookies[0] = RegClientCookie("VIP_Chat_Prefix", "VIP_Chat_Prefix", CookieAccess_Private);
	g_hCookies[1] = RegClientCookie("VIP_Chat_PrefixColor", "VIP_Chat_PrefixColor", CookieAccess_Private);
	g_hCookies[2] = RegClientCookie("VIP_Chat_NameColor", "VIP_Chat_NameColor", CookieAccess_Private);
	g_hCookies[3] = RegClientCookie("VIP_Chat_TextColor", "VIP_Chat_TextColor", CookieAccess_Private);

	LoadTranslations("vip_chat.phrases");

	if(VIP_IsVIPLoaded())
	{
		VIP_OnVIPLoaded();
	}
}

public void OnMapStart()
{
	char szBuffer[256];

	if(g_hKeyValues != INVALID_HANDLE)
	{
		CloseHandle(g_hKeyValues);
	}

	g_hColorsTrie.Clear();

	g_hKeyValues = CreateKeyValues("Chat");
	BuildPath(Path_SM, szBuffer, 256, "data/vip/modules/chat_config.ini");

	if (FileToKeyValues(g_hKeyValues, szBuffer) == false)
	{
		CloseHandle(g_hKeyValues);
		SetFailState("Не удалось открыть файл \"%s\"", szBuffer);
	}

	KvRewind(g_hKeyValues);
	if(KvJumpToKey(g_hKeyValues, "Settings"))
	{
		g_bIgnoreTriggers = !!KvGetNum(g_hKeyValues, "ignore_chat_triggers");
		g_bIgnorePhrases = !!KvGetNum(g_hKeyValues, "ignore_chat_phrases");
		g_bReplaceOther = !!KvGetNum(g_hKeyValues, "replace_other");
	}
	if(g_hIgnoredPhrases != INVALID_HANDLE)
	{
		CloseHandle(g_hIgnoredPhrases);
	}

	g_hIgnoredPhrases = CreateArray(ByteCountToCells(64));

	KvRewind(g_hKeyValues);
	if(KvJumpToKey(g_hKeyValues, "Ignore"))
	{
		if(KvGotoFirstSubKey(g_hKeyValues, false))
		{
			do
			{
				KvGetString(g_hKeyValues, NULL_STRING, szBuffer, sizeof(szBuffer));
				PushArrayString(g_hIgnoredPhrases, szBuffer);
			} while (KvGotoNextKey(g_hKeyValues, false));
		}
	}
	
	if(!GetArraySize(g_hIgnoredPhrases))
	{
		CloseHandle(g_hIgnoredPhrases);
		g_hIgnoredPhrases = null;
	}

	LoadColors("NameColor_List");
	LoadColors("TextColor_List");
	LoadColors("PrefixColor_List");
}

void LoadColors(const char[] szKey)
{
	KvRewind(g_hKeyValues);
	if(KvJumpToKey(g_hKeyValues, szKey) && KvGotoFirstSubKey(g_hKeyValues, false))
	{
		char sColorCode[16], sColorName[32];
		do
		{
			KvGetSectionName(g_hKeyValues, sColorName, sizeof(sColorName));
			KvGetString(g_hKeyValues, NULL_STRING, sColorCode, sizeof(sColorCode));
			SetTrieString(g_hColorsTrie, sColorCode, sColorName);
		} while (KvGotoNextKey(g_hKeyValues, false));
	}
}

#if defined USE_SCP
public Action OnChatMessage(int &iClient, ArrayList hRecipients, char[] szName, char[] szMessage)
#else
public Action CP_OnChatMessage(int &iClient, ArrayList hRecipients, char[] szFlags, char[] szName, char[] szMessage, bool& bProcesscolors, bool& bRemovecolors)
#endif
{
	if(VIP_IsClientVIP(iClient) && VIP_IsClientFeatureUse(iClient, g_szFeature[0]))
	{
		if(g_bIgnoreTriggers &&
			(szMessage[0] == '!' ||
			szMessage[0] == '/' ||
			szMessage[0] == '@'))
		{
			return Plugin_Continue;
		}

		if(g_bIgnorePhrases && g_hIgnoredPhrases && FindStringInArray(g_hIgnoredPhrases, szMessage) != -1)
		{
			return Plugin_Continue;
		}

		char szColor[64], szBuffer[256], szNickName[256];
		if(GetClientChat(iClient, TEXT_COLOR, szColor, sizeof(szColor)))
		{
			Format(szMessage, MAXLENGTH_MESSAGE, "%s%s", szColor, szMessage);
		}

		if(g_bReplaceOther)
		{
			GetClientName(iClient, szNickName, sizeof(szNickName));
		}
		else
		{
			strcopy(szNickName, sizeof(szNickName), szName);
		}

		if(GetClientChat(iClient, NAME_COLOR, szColor, sizeof(szColor)))
		{
			Format(szNickName, sizeof(szNickName), "%s%s", szColor, szNickName);
		}
		else
		{
			Format(szNickName, sizeof(szNickName), "\x03%s", szNickName);
		}

		if(GetClientChat(iClient, PREFIX, szBuffer, sizeof(szBuffer)))
		{
			Format(szNickName, sizeof(szNickName), "%s %s", szBuffer, szNickName);

			if(GetClientChat(iClient, PREFIX_COLOR, szColor, sizeof(szColor)))
			{
				Format(szNickName, sizeof(szNickName), "%s%s", szColor, szNickName);
			}
		}
		
		if(g_EngineVersion == Engine_CSGO)
		{
			Format(szNickName, sizeof(szNickName), " %s", szNickName);
		}

		ReplaceStringColors(szNickName, sizeof(szNickName));
		strcopy(szName, MAXLENGTH_NAME, szNickName);
		ReplaceStringColors(szMessage, MAXLENGTH_MESSAGE);
		return Plugin_Changed;
	}

	return Plugin_Continue;
}


/*
if(index == PREFIX)
	{
		KvJumpToKey(g_hKeyValues, "Prefixes");
	}
	else
	{
		KvJumpToKey(g_hKeyValues, "Colors");
	}
	
	VIP_GetClientFeatureString(iClient, g_szFeatures[index], szBuffer, sizeof(szBuffer));
	
	if(strlen(sClientTag) < 5 || !KvJumpToKey(g_hKeyValues, sClientTag[5]))
	{
		KvJumpToKey(g_hKeyValues, "default");
	}

	szBuffer[0] = 0;
	
	if(KvGotoFirstSubKey(g_hKeyValues, false))
	{
	*/
	
bool GetClientChat(int iClient, int index, char[] szBuffer, int iMaxLen)
{
	if(VIP_IsClientFeatureUse(iClient, g_szFeatures[index]))
	{
		VIP_GetClientFeatureString(iClient, g_szFeatures[index], szBuffer, iMaxLen);
		if(strcmp(szBuffer, g_szCUSTOM) == 0 || strncmp(szBuffer, g_szLIST, 4) == 0)
		{
			GetClientCookie(iClient, g_hCookies[index], szBuffer, iMaxLen);
		}
		else
		{
			char sCookie[4];
			GetClientCookie(iClient, g_hCookies[index], sCookie, sizeof(sCookie));
			if(sCookie[0] == '0')
			{
				return false;
			}
		}

		if(szBuffer[0] == '0')
		{
			return false;
		}

		if(szBuffer[0])
		{
			return true;
		}
	}

	return false;
}

int ReplaceStringColors(char[] szMessage, int iMaxLen)
{
	ReplaceString(szMessage, iMaxLen, "{DEFAULT}",	"\x01", false);
	ReplaceString(szMessage, iMaxLen, "{TEAM}",		"\x03", false);
	ReplaceString(szMessage, iMaxLen, "{GREEN}",		"\x04", false);
	ReplaceString(szMessage, iMaxLen, "#",			"\x07", false);

	if(g_EngineVersion == Engine_CSGO)
	{
		ReplaceString(szMessage, iMaxLen, "{RED}",			"\x02", false);
		ReplaceString(szMessage, iMaxLen, "{LIME}",			"\x05", false);
		ReplaceString(szMessage, iMaxLen, "{LIGHTGREEN}",	"\x06", false);
		ReplaceString(szMessage, iMaxLen, "{LIGHTRED}",		"\x07", false);
		ReplaceString(szMessage, iMaxLen, "{GRAY}",			"\x08", false);
		ReplaceString(szMessage, iMaxLen, "{LIGHTOLIVE}",	"\x09", false);
		ReplaceString(szMessage, iMaxLen, "{OLIVE}",			"\x10", false);
		ReplaceString(szMessage, iMaxLen, "{PURPLE}",		"\x0E", false);
		ReplaceString(szMessage, iMaxLen, "{LIGHTBLUE}",		"\x0B", false);
		ReplaceString(szMessage, iMaxLen, "{BLUE}",			"\x0C", false);
	}
	else if(g_EngineVersion == Engine_CSS)
	{
		ReplaceString(szMessage, iMaxLen, "{DARKGREEN}",	"\x05", false);
	}
}

public int OnDrawItem(int iClient, const char[] szFeatureName, int iStyle)
{
	switch(VIP_GetClientFeatureStatus(iClient, g_szFeature[0]))
	{
		case ENABLED: return ITEMDRAW_DEFAULT;
		case DISABLED: return ITEMDRAW_DISABLED;
		case NO_ACCESS: return ITEMDRAW_RAWLINE;
	}

	return iStyle;
}

public bool OnSelectItem(int iClient, const char[] szFeatureName)
{
	DisplayChatMainMenu(iClient);

	return false;
}

void DisplayChatMainMenu(int iClient)
{
//	char szBuffer[128], Handle:hMenu;
	Menu hMenu = CreateMenu(ChatMainMenu_Handler);
	SetMenuExitBackButton(hMenu, true);
	SetMenuTitle(hMenu, "%T:\n ", "MainMenuTitle", iClient);
/*
	FormatEx(szBuffer, sizeof(szBuffer), "%T\n ", "DisableAll");
	AddMenuItem(hMenu, "", szBuffer);
*/
	AddMenuFeatureItem(iClient, PREFIX, hMenu, "Prefix");
	AddMenuFeatureItem(iClient, PREFIX_COLOR, hMenu, "PrefixColor");
	AddMenuFeatureItem(iClient, NAME_COLOR, hMenu, "NameColor");
	AddMenuFeatureItem(iClient, TEXT_COLOR, hMenu, "TextColor");

	hMenu.Display(iClient, MENU_TIME_FOREVER);
}

void AddMenuFeatureItem(int iClient, int index, Menu hMenu, const char[] szFeatureName)
{
	char szBuffer[128];
	if(VIP_IsClientFeatureUse(iClient, g_szFeatures[index]))
	{
		char sItemInfo[128];
		VIP_GetClientFeatureString(iClient, g_szFeatures[index], szBuffer, sizeof(szBuffer));
		GetClientCookie(iClient, g_hCookies[index], sItemInfo, sizeof(sItemInfo));
		
		if(sItemInfo[0] == '0')
		{
			FormatEx(szBuffer, sizeof(szBuffer), "%T [%T]", szFeatureName, iClient, "Disabled", iClient);
		}
		else
		{
			if(strcmp(szBuffer, g_szCUSTOM) == 0 || strncmp(szBuffer, g_szLIST, 4) == 0)
			{
				if(sItemInfo[0])
				{
					GetTrieString(g_hColorsTrie, sItemInfo, sItemInfo, sizeof(sItemInfo));
					FormatEx(szBuffer, sizeof(szBuffer), "%T - %s", szFeatureName, iClient, sItemInfo);
				}
				else
				{
					FormatEx(szBuffer, sizeof(szBuffer), "%T [%T]", szFeatureName, iClient, "NotChosen", iClient);
				}
			}
			else
			{
				Format(szBuffer, sizeof(szBuffer), "%T - %s", szFeatureName, iClient, szBuffer);
			}
		}

		FormatEx(sItemInfo, sizeof(sItemInfo), "%d_%s", index, szFeatureName);
		AddMenuItem(hMenu, sItemInfo, szBuffer);
	}
	else
	{
		FormatEx(szBuffer, sizeof(szBuffer), "%T (%T)", szFeatureName, iClient, "NoAccess", iClient);
		AddMenuItem(hMenu, "", szBuffer, ITEMDRAW_DISABLED);
	}
}

public int ChatMainMenu_Handler(Menu hMenu, MenuAction eAction, int iClient, int iItem)
{
	switch(eAction)
	{
		case MenuAction_End: CloseHandle(hMenu);
		case MenuAction_Cancel:
		{
			if(iItem == MenuCancel_ExitBack) VIP_SendClientVIPMenu(iClient);
		}
		case MenuAction_Select:
		{
			char szBuffer[128], sItemInfo[128];
			int index;
			GetMenuItem(hMenu, iItem, sItemInfo, sizeof(sItemInfo));
			
			index = sItemInfo[0]-48;

			VIP_GetClientFeatureString(iClient, g_szFeatures[index], szBuffer, sizeof(szBuffer));
			StringMap hTrie = VIP_GetVIPClientTrie(iClient);

			hTrie.SetString("Chat_MenuType", sItemInfo[2]);
			hTrie.SetValue("Chat_CookieIndex", index);

			if(strcmp(szBuffer, g_szCUSTOM) == 0)
			{
				GetClientCookie(iClient, g_hCookies[index], sItemInfo, sizeof(sItemInfo));
				if(sItemInfo[0] == '0')
				{
					sItemInfo[0] = 0;
				}

				DisplayWaitChatMenu(iClient, sItemInfo, false, index);
			}
			else if(strncmp(szBuffer, g_szLIST, 4) == 0)
			{
				DisplayChatListMenu(iClient, sItemInfo[2], index);
			}
			else
			{
				RemoveFromTrie(hTrie, "Chat_MenuType");
				RemoveFromTrie(hTrie, "Chat_CookieIndex");

				GetClientCookie(iClient, g_hCookies[index], sItemInfo, sizeof(sItemInfo));
				
				bool bEnable;
				if(sItemInfo[0])
				{
					bEnable = !!StringToInt(sItemInfo);
				}
				else
				{
					bEnable = true;
				}

				bEnable = !bEnable;
				SetClientCookie(iClient, g_hCookies[index], bEnable ? "":"0");
				
				GetClientCookie(iClient, g_hCookies[index], sItemInfo, sizeof(sItemInfo));

				DisplayChatMainMenu(iClient);
			}
		}
	}
	
	return 0;
}

void DisplayChatListMenu(int iClient, const char[] szKey, int index)
{
	char szBuffer[128], szClientColor[64];
	Menu hMenu;
	hMenu = CreateMenu(ChatListMenu_Handler);
	SetMenuExitBackButton(hMenu, true);

	SetMenuTitle(hMenu, "%T:\n ", szKey, iClient);
	GetClientCookie(iClient, g_hCookies[index], szClientColor, sizeof(szClientColor));

	if(szClientColor[0] && szClientColor[0] != '0')
	{
		FormatEx(szBuffer, sizeof(szBuffer), "%T\n ", "Disable", iClient);
		AddMenuItem(hMenu, "_disable", szBuffer);
	}

	KvRewind(g_hKeyValues);
	
	if(index == PREFIX)
	{
		KvJumpToKey(g_hKeyValues, "Prefixes");
	}
	else
	{
		KvJumpToKey(g_hKeyValues, "Colors");
	}
	
	VIP_GetClientFeatureString(iClient, g_szFeatures[index], szBuffer, sizeof(szBuffer));
	
	if(strlen(szBuffer) < 5 || !KvJumpToKey(g_hKeyValues, szBuffer[5]))
	{
		KvJumpToKey(g_hKeyValues, "default");
	}

	szBuffer[0] = 0;
	
	if(KvGotoFirstSubKey(g_hKeyValues, false))
	{
		char szValue[64];
		do
		{
			KvGetString(g_hKeyValues, NULL_STRING, szValue, sizeof(szValue));
			KvGetSectionName(g_hKeyValues, szBuffer, sizeof(szBuffer));
			if(strcmp(szClientColor, szValue) == 0)
			{
				Format(szBuffer, sizeof(szBuffer), "%s [X]", szBuffer);
				AddMenuItem(hMenu, szValue, szBuffer, ITEMDRAW_DISABLED);
				continue;
			}

			AddMenuItem(hMenu, szValue, szBuffer);
		}
		while (KvGotoNextKey(g_hKeyValues, false));
	}

	if(szBuffer[0] == 0)
	{
		FormatEx(szBuffer, sizeof(szBuffer), "%T", "NoItems", iClient);
		AddMenuItem(hMenu, "", szBuffer, ITEMDRAW_DISABLED);
	}

	DisplayMenu(hMenu, iClient, MENU_TIME_FOREVER);
}

public int ChatListMenu_Handler(Menu hMenu, MenuAction eAction, int iClient, int iItem)
{
	switch(eAction)
	{
		case MenuAction_End: CloseHandle(hMenu);
		case MenuAction_Cancel:
		{
			if(iItem == MenuCancel_ExitBack) DisplayChatMainMenu(iClient);
		}
		case MenuAction_Select:
		{
			char szValue[64], sColorName[128];
			StringMap hTrie;
			int index;
			GetMenuItem(hMenu, iItem, szValue, sizeof(szValue), _, sColorName, sizeof(sColorName));

			hTrie = VIP_GetVIPClientTrie(iClient);
			GetTrieValue(hTrie, "Chat_CookieIndex", index);
			
			if(strcmp(szValue, "_disable") == 0)
			{
				SetClientCookie(iClient, g_hCookies[index], "0");
				RemoveFromTrie(hTrie, "Chat_MenuType");
				RemoveFromTrie(hTrie, "Chat_CookieIndex");
				DisplayChatMainMenu(iClient);
				return 0;
			}
			
			char szBuffer[64];

			SetClientCookie(iClient, g_hCookies[index], szValue);

			GetTrieString(hTrie, "Chat_MenuType", szBuffer, sizeof(szBuffer));

			if(index == PREFIX)
			{
				VIP_PrintToChatClient(iClient, "\x03%T %T: \x04%s", "Set", iClient, szBuffer, iClient, sColorName);
			}
			else
			{
				VIP_PrintToChatClient(iClient, "\x03%T -> %s%T", "Set", iClient, szValue, szBuffer, iClient);
			}

			DisplayChatListMenu(iClient, szBuffer, index);
		}
	}
	
	return 0;
}

void DisplayWaitChatMenu(int iClient, const char[] szValue = "", const bool bIsValid = false, int index)
{
	g_bWaitChat[iClient] = true;

	Menu hMenu = CreateMenu(WaitChatMenu_Handler);

	if(szValue[0])
	{
		SetMenuTitle(hMenu, "%T \"%T\"\n%T: %s\n ", "EnterValueInChat", iClient, "Confirm", iClient, "Value", iClient, szValue);
	}
	else
	{
		SetMenuTitle(hMenu, "%T \"%T\"\n ", "EnterValueInChat", iClient, "Confirm", iClient);
	}
	
	char szBuffer[128];

	FormatEx(szBuffer, sizeof(szBuffer), "%T", "Confirm", iClient);
	AddMenuItem(hMenu, szValue, szBuffer, bIsValid ? ITEMDRAW_DEFAULT:ITEMDRAW_DISABLED);

	FormatEx(szBuffer, sizeof(szBuffer), "%T\n ", "Cancel", iClient);
	AddMenuItem(hMenu, "", szBuffer);
	
	GetClientCookie(iClient, g_hCookies[index], szBuffer, sizeof(szBuffer));

	if(szBuffer[0] && szBuffer[0] != '0')
	{
		FormatEx(szBuffer, sizeof(szBuffer), "%T\n ", "Disable", iClient);
		AddMenuItem(hMenu, "_disable", szBuffer);
	}

	KvRewind(g_hKeyValues);
	if(KvJumpToKey(g_hKeyValues, "Help"))
	{
		FormatEx(szBuffer, sizeof(szBuffer), "%T\n ", "Help", iClient);
		AddMenuItem(hMenu, "_help", szBuffer);
	}
	else
	{
		AddMenuItem(hMenu, "", "", ITEMDRAW_NOTEXT);
	}

	AddMenuItem(hMenu, "", "", ITEMDRAW_NOTEXT);
	AddMenuItem(hMenu, "", "", ITEMDRAW_NOTEXT);

	DisplayMenu(hMenu, iClient, MENU_TIME_FOREVER);
}

public int WaitChatMenu_Handler(Menu hMenu, MenuAction eAction, int iClient, int iItem)
{
	switch(eAction)
	{
		case MenuAction_End: CloseHandle(hMenu);
		case MenuAction_Cancel:
		{
			g_bWaitChat[iClient] = false;
			if(iItem == MenuCancel_ExitBack)
			{
				DisplayChatMainMenu(iClient);
			}
		}
		case MenuAction_Select:
		{
			StringMap hTrie;
			int index;
			hTrie = VIP_GetVIPClientTrie(iClient);
			GetTrieValue(hTrie, "Chat_CookieIndex", index);

			if(iItem == 0)
			{
				char szBuffer[64], szColor[64];
				GetMenuItem(hMenu, iItem, szColor, sizeof(szColor));

				SetClientCookie(iClient, g_hCookies[index], szColor);

				GetTrieString(hTrie, "Chat_MenuType", szBuffer, sizeof(szBuffer));

				if(index == PREFIX)
				{
					VIP_PrintToChatClient(iClient, "\x03%T %T: \x04%s", "Set", iClient, szBuffer, iClient, szColor);
				}
				else
				{
					VIP_PrintToChatClient(iClient, "\x03%T -> %s%T", "Set", iClient, szColor, szBuffer, iClient);
				}
			}
			else
			{
				char szBuffer[64];
				GetMenuItem(hMenu, iItem, szBuffer, sizeof(szBuffer));
				if(strcmp(szBuffer, "_disable") == 0)
				{
					SetClientCookie(iClient, g_hCookies[index], "0");
				}
				else if(strcmp(szBuffer, "_help") == 0)
				{
					DisplayHelpMenu(iClient);
					return 0;
				}
			}
			
			RemoveFromTrie(hTrie, "Chat_MenuType");
			RemoveFromTrie(hTrie, "Chat_CookieIndex");
			g_bWaitChat[iClient] = false;
			DisplayChatMainMenu(iClient);
		}
	}
	
	return 0;
}

public Action OnClientSayCommand(int iClient, const char[] sCommand, const char[] sArgs)
{
	if(iClient > 0 && iClient <= MaxClients && sArgs[0] && g_bWaitChat[iClient] && IsClientInGame(iClient))
	{
		char szText[64];
		strcopy(szText, sizeof(szText), sArgs);
		TrimString(szText);
		StripQuotes(szText);
		
		if(szText[0])
		{
			int index;
			if(GetTrieValue(VIP_GetVIPClientTrie(iClient), "Chat_CookieIndex", index))
			{
				DisplayWaitChatMenu(iClient, szText, true, index);
			}
			else
			{
				g_bWaitChat[iClient] = false;
			}
		}

		return Plugin_Handled;
	}

	return Plugin_Continue;
}

void DisplayHelpMenu(int iClient)
{
	char szBuffer[128];
	
	Panel hPanel = CreatePanel();
	FormatEx(szBuffer, sizeof(szBuffer), "%T:\n ", "Help");
	hPanel.SetTitle(szBuffer);

	KvRewind(g_hKeyValues);
	if(KvJumpToKey(g_hKeyValues, "Help"))
	{
		if(KvGotoFirstSubKey(g_hKeyValues, false))
		{
			do
			{
				KvGetString(g_hKeyValues, NULL_STRING, szBuffer, sizeof(szBuffer));
				DrawPanelText(hPanel, szBuffer);
			} while (KvGotoNextKey(g_hKeyValues, false));
		}
	}

	DrawPanelItem(hPanel, " ", ITEMDRAW_SPACER|ITEMDRAW_RAWLINE);

	DrawPanelItem(hPanel, "<-");

	SendPanelToClient(hPanel, iClient, ChatInfoMenu_Handler, MENU_TIME_FOREVER); 
	CloseHandle(hPanel); 
}

public int ChatInfoMenu_Handler(Menu hMenu, MenuAction eAction, int iClient, int iItem)
{
	if(eAction == MenuAction_Select)
	{
		int index;
		char szBuffer[64];
		VIP_GetVIPClientTrie(iClient).GetValue("Chat_CookieIndex", index);
		GetClientCookie(iClient, g_hCookies[index], szBuffer, sizeof(szBuffer));
		if(szBuffer[0] == '0')
		{
			szBuffer[0] = 0;
		}

		DisplayWaitChatMenu(iClient, szBuffer, false, index);
	}
	
	return 0;
}
